# coding: utf-8
from future import standard_library
standard_library.install_aliases()
from future.builtins import str
import uuid

import six
from m3 import ApplicationLogicException

from kinder.core.children.models import DelegateTypeEnumerate, GenderEnumerate
from kinder.core.declaration.models import DeclarationPrivilege
from kinder.core.dict.models import DULDelegateType, GroupSpec
from kinder.core.queue_module.context import QueueContext
from kinder.core.queue_module.queue import QueueSingleton
from kinder.core.unit.models import Unit, UnitKind
from kinder.plugins.contingent.models import DelegateContingent

if six.PY3:
    from io import StringIO
else:
    from io import StringIO

MALE = 'Male'  # мужчина
FEMALE = 'Female'  # женщина
# В задаче EDUKNDG-7390 указано, что для услуги GetApplication
# это зафиксированное значение
GETAPP_SERVICE_TYPE = '-10002202018'

DELEGATE_TYPE_CONC = {
    DelegateTypeEnumerate.MOTHER: (1, u'Мать'),
    DelegateTypeEnumerate.FATHER: (2, u'Отец'),
    DelegateTypeEnumerate.LEX: (3, u'Опекун'),

}


class CustomStringIO(StringIO):
    """Обход UnicodeDecodeError: 'ascii' codec can't decode
    при сливание строк
    """

    def getvalue(self):
        """
        """

        def _complain_ifclosed(closed):
            if closed:
                raise ValueError("I/O operation on closed file")

        _complain_ifclosed(self.closed)
        if self.buflist:
            for line in self.buflist:
                if isinstance(line, str):
                    line = str(line, 'utf-8')
                self.buf += line
            self.buflist = []
        return self.buf


def export_response(response):
    """Рендерит классы из kinder_conc.py в xml

    :param response: Инстанс класса kinder_conc.GeneratedsSuper
    :return:
    """
    mystdout = CustomStringIO()
    response.export(mystdout, 0, name_='FormDataResponse', pretty_print=False)
    return mystdout.getvalue()


def get_queue_info_short(declaration, unit_id, external_context=None):
    """Получение позиции очереди всего в очереди
    :param declaration: Заявление
    :param unit_id:  int Организация, по которой формируется очередь
    :param external_context: dict доп параметры для контекста очереди
    :return:
    """

    context = QueueContext.make_base_context(
        declaration.children.date_of_birth, unit_id)
    if external_context:
        for key, value in external_context.items():
            setattr(context, key, value)

    queue = QueueSingleton().get_class()(context)
    result = (None, None)

    rows, all = queue.get_list(0)
    for row in rows:
        if row['id'] == declaration.id:
            result = (row['id'], all)
            break
    return result


def get_spec_decl(declaration):
    """Возвращаем специфику групп в заявление
    :param declaration:
    :return: идентификатор из справочник специфик
    :rtype: str
    """
    default = GroupSpec.objects.get(code=GroupSpec.DEFAULT_CODE)
    return str(declaration.spec_id) or str(default.id)


def get_oktmo_region():
    """Возвращает ОКТМО головного учреждения
    :return: октмо или None
    """
    region = Unit.objects.filter(kind=UnitKind.REGION).last()
    if not region:
        return None
    return region.octmo


def get_attr(record, attr, default=None):
    """Геттер параметра из объекта
    Проверяет что объект не пуст
    :param record: объект
    :param attr: имя атрибута
    :param default: дефолтное значение
    :return: значение атрибута
    """
    if record:
        return getattr(record, attr, default)
    else:
        return None


def get_declaration_units(declaration):
    """
    возвращаем желаемые организации заявки
    в виде списка [('unit_name', 'unit_id', 'ord'), ...]

    :param declaration: заявка
    :return:
    """
    return declaration.declarationunit_set.values_list(
        'unit__name', 'unit_id', 'ord')


def get_dul_delegate_type(delegate):
    """Возвращает  наименование Типы документов удостоверяющего
    личность представителя
    :param delegate: Delegate
    :return:
    """
    try:
        record = DULDelegateType.objects.get(id=delegate.dul_type_id)
    except DULDelegateType.DoesNotExist:
        return None
    return record.name


def get_contingent_param_delegate(delegate):
    """Возвращает сведения гражданства из плагина контингент представителя
    :param delegate: Delegate
    :return: tuple
    """
    try:
        record = DelegateContingent.objects.get(delegate=delegate.id)
    except DelegateContingent.DoesNotExist:
        return None, None, None
    return record.citizenship, record.citizenship_country, record.birthplace


def get_privileges(declaration):
    """Возвращает параметры льгот в заявление
    :param declaration: Declaration
    :return:  list список параметров льготы
    """
    return DeclarationPrivilege.objects.filter(
        declaration=declaration).values_list(
        'privilege_id',
        'privilege__name',
        'doc_type_id',
        'doc_type__name',
        'doc_series',
        'doc_number'
    )


def type_mapping(delegate):
    """Маппинг типа представителя в значения концентратора
     :param delegate: Delegate
    :return: tuple (code, name)
    """
    return DELEGATE_TYPE_CONC[delegate.type]


def gender_mapping(delegate):
    """Маппинг в значения для концентратора пола предствителя
    :param delegate: Delegate
    :return: tuple
    """
    try:
        record = DelegateContingent.objects.get(delegate=delegate.id)
    except DelegateContingent.DoesNotExist:
        return FEMALE

    if record.gender ==  DelegateTypeEnumerate.MOTHER:
        return FEMALE
    else:
        return MALE


def gender_mapping_child(child):
    """Маппинг в значения для концентратора пола предствителя
    :param child: Children
    :return: str
    """
    if child.gender == GenderEnumerate.FEMALE:
        return FEMALE
    else:
        return MALE


def get_order_id(declaration):
    """Получение идентификатора заявки."""
    try:
        _id = int(declaration.client_id)
    except (ValueError, TypeError):
        raise ApplicationLogicException(u'Идентификатор заявки невалиден')

    return _id


def generate_message_id():
    """Генерация ID сообщения."""
    return str(uuid.uuid1())
